﻿Imports System.Data.Entity
Imports Model
Imports Model.Interfaces

Public Class OrdersDbContext
    Inherits DbContext

    Public Overridable Property Orders As DbSet(Of Model.Order)
    Public Overridable Property WorkCenters As DbSet(Of Model.WorkCenter)
    Public Overridable Property Answers As DbSet(Of Model.Answer)


    Public Sub New()
        MyBase.New("OrdersContext")
        Database.SetInitializer(New MigrateDatabaseToLatestVersion(Of OrdersDbContext, Configuration))
        'Database.Initialize(True)

        'MyBase.Database.Log = Sub(s) Debug.WriteLine(String.Format("{0}: {1}", DateTime.Now.ToLongTimeString & ":" & DateTime.Now.Millisecond.ToString, s))

    End Sub


    Public Overrides Function SaveChanges() As Integer
        UpdateEntrys()
        Return MyBase.SaveChanges()
    End Function

    Public Overrides Function SaveChangesAsync() As Task(Of Integer)
        UpdateEntrys()
        Return MyBase.SaveChangesAsync()
    End Function


    Private Sub UpdateEntrys()
        Dim objectStateEntries = ChangeTracker.Entries().Where(Function(e) TypeOf e.Entity Is ModelBase AndAlso e.State <> EntityState.Detached AndAlso e.State <> EntityState.Unchanged).ToList()
        For Each eEntry In objectStateEntries
            Dim entityBase = TryCast(eEntry.Entity, ModelBase)
            If entityBase Is Nothing Then Continue For
            Select Case eEntry.State
                Case EntityState.Deleted
                    If TypeOf entityBase Is ILogicalDelete Then
                        eEntry.State = EntityState.Modified
                        CType(entityBase, ILogicalDelete).DeletedTimestamp = DateTime.Now
                        CType(entityBase, ILogicalDelete).DeletedFlag = True
                    End If
                Case EntityState.Modified
                    CType(entityBase, ModelBase).LastUpdateBy = Environment.UserName
                    CType(entityBase, ModelBase).LastUpdateOn = Environment.MachineName
                    If TypeOf entityBase Is ILogicalTimestamp Then
                        CType(entityBase, ILogicalTimestamp).LastUpdateTimestamp = DateTime.Now
                    End If

                Case EntityState.Added
                    CType(entityBase, ModelBase).CreatedBy = Environment.UserName
                    CType(entityBase, ModelBase).CreatedOn = Environment.MachineName
                    If TypeOf entityBase Is ILogicalTimestamp Then
                        CType(entityBase, ILogicalTimestamp).CreationTimestamp = DateTime.Now
                        CType(entityBase, ILogicalTimestamp).LastUpdateTimestamp = DateTime.Now

                    End If

            End Select
        Next
    End Sub

End Class
